@@ -3,6 +3,8 @@
 #include <string.h>
 #include <mp4p/mp4p.h>
 
+static __thread int _dbg_indent = 0;
+
 static mp4p_atom_t *
 _atom_load (mp4p_atom_t *parent_atom, mp4p_file_callbacks_t *fp);
 
@@ -37,8 +39,6 @@ mp4p_atom_type_compare (mp4p_atom_t *atom, const char *type) {
     return mp4p_fourcc_compare(atom->type, type);
 }
 
-static int _dbg_indent = 0;
-
 static void
 _dbg_print_fourcc (const char *fourcc) {
     printf ("%c%c%c%c", fourcc[0], fourcc[1], fourcc[2], fourcc[3]);
@@ -53,12 +53,10 @@ _dbg_print_indent (void) {
 
 static void
 _dbg_print_atom (mp4p_atom_t *atom) {
-#if 1
     _dbg_print_indent();
     _dbg_print_fourcc(atom->type);
     printf (" pos=%x size=%x", (int)atom->pos, (int)atom->size);
     printf ("\n");
-#endif
 }
 
 static int
@@ -68,7 +66,9 @@ _load_subatoms (mp4p_atom_t *atom, mp4p_file_callbacks_t *fp) {
     while (fp->tell (fp) < atom->pos + atom->size) {
         mp4p_atom_t *c = _atom_load (atom, fp);
         if (!c) {
-            return -1;
+// FIXME: add proper error reporting
+//            fprintf (stderr, "Failed to read child atom\n");
+            break;
         }
         if (!atom->subatoms) {
             atom->subatoms = tail = c;
@@ -125,7 +125,7 @@ _read_uint32 (mp4p_file_callbacks_t *fp, uint32_t *value) {
 #define FREE_ATOM_BUFFER() free (atombuf);
 
 #define ATOM_DEF_INNER(atomname,headersize)\
-    mp4p_##atomname##_t *atom_data = calloc (sizeof (mp4p_##atomname##_t), 1);\
+    mp4p_##atomname##_t *atom_data = calloc (1, sizeof (mp4p_##atomname##_t));\
     atom->data = atom_data;\
     atom->free = mp4p_##atomname##_atomdata_free;\
     atom->write = (mp4p_atom_data_write_func_t)mp4p_##atomname##_atomdata_write;\
@@ -273,6 +273,16 @@ mp4p_atom_init (mp4p_atom_t *parent_atom, mp4p_atom_t *atom, mp4p_file_callbacks
     ATOM_DEF(chpl)
     ATOM_DEF(chap)
     else {
+        // don't allow weird atom types with out of range characters
+        for (int i = 0; i < 4; i++) {
+            if (atom->type[i] != COPYRIGHT_SYM[0] && atom->type[i] != ' ' && (atom->type[i] < '-' || atom->type[i] > 'z')) {
+                return -1;
+            }
+        }
+        // don't load large atoms into RAM
+        if (atom->size > 2*1024*1024) {
+            return 0;
+        }
         atom->data = malloc (atom->size - 8);
         atom->free = free;
         READ_BUF(fp, atom->data, atom->size - 8);
@@ -294,7 +304,7 @@ static mp4p_atom_t *
 _atom_load (mp4p_atom_t *parent_atom, mp4p_file_callbacks_t *fp) {
     size_t fpos = fp->tell (fp);
 
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
 
     atom->pos = fpos;
 
@@ -729,8 +739,8 @@ mp4p_genre_index_for_name (const char *name) {
 
 mp4p_atom_t *
 mp4p_ilst_create_custom (const char *name, const char *text) {
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
-    mp4p_ilst_meta_t *meta = calloc (sizeof (mp4p_ilst_meta_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
+    mp4p_ilst_meta_t *meta = calloc (1, sizeof (mp4p_ilst_meta_t));
     meta->custom = 1;
     atom->data = meta;
     atom->free = mp4p_ilst_meta_atomdata_free;
@@ -750,8 +760,8 @@ mp4p_ilst_create_custom (const char *name, const char *text) {
 
 mp4p_atom_t *
 mp4p_ilst_create_genre (const char *text) {
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
-    mp4p_ilst_meta_t *meta = calloc (sizeof (mp4p_ilst_meta_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
+    mp4p_ilst_meta_t *meta = calloc (1, sizeof (mp4p_ilst_meta_t));
     atom->data = meta;
     atom->free = mp4p_ilst_meta_atomdata_free;
     atom->write = (mp4p_atom_data_write_func_t)mp4p_ilst_meta_atomdata_write;
@@ -777,8 +787,8 @@ mp4p_ilst_create_genre (const char *text) {
 
 mp4p_atom_t *
 mp4p_ilst_create_track_disc (const char *type, uint16_t index, uint16_t total) {
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
-    mp4p_ilst_meta_t *meta = calloc (sizeof (mp4p_ilst_meta_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
+    mp4p_ilst_meta_t *meta = calloc (1, sizeof (mp4p_ilst_meta_t));
     atom->data = meta;
     atom->free = mp4p_ilst_meta_atomdata_free;
     atom->write = (mp4p_atom_data_write_func_t)mp4p_ilst_meta_atomdata_write;
@@ -797,8 +807,8 @@ mp4p_ilst_create_track_disc (const char *type, uint16_t index, uint16_t total) {
 
 mp4p_atom_t *
 mp4p_ilst_create_text (const char *type, const char *text) {
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
-    mp4p_ilst_meta_t *meta = calloc (sizeof (mp4p_ilst_meta_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
+    mp4p_ilst_meta_t *meta = calloc (1, sizeof (mp4p_ilst_meta_t));
     atom->data = meta;
     atom->free = mp4p_ilst_meta_atomdata_free;
     atom->write = (mp4p_atom_data_write_func_t)mp4p_ilst_meta_atomdata_write;
@@ -857,7 +867,7 @@ mp4p_atom_remove_subatom (mp4p_atom_t *atom, mp4p_atom_t *subatom) {
 
 mp4p_atom_t *
 mp4p_atom_new (const char *type) {
-    mp4p_atom_t *atom = calloc (sizeof (mp4p_atom_t), 1);
+    mp4p_atom_t *atom = calloc (1, sizeof (mp4p_atom_t));
     memcpy (atom->type, type, 4);
     return atom;
 }
@@ -904,6 +914,16 @@ mp4p_atom_clone_list (mp4p_atom_t *src) {
     return clone;
 }
 
+mp4p_atom_t *
+mp4p_meta_create_atom (void) {
+    mp4p_atom_t *atom = mp4p_atom_new ("meta");
+    atom->write = (mp4p_atom_data_write_func_t)mp4p_meta_atomdata_write;
+    atom->write_data_before_subatoms = 1;
+    atom->data = calloc (4, 1);
+    atom->free = mp4p_meta_atomdata_free;
+    return atom;
+}
+
 void
 mp4p_atom_update_size (mp4p_atom_t *atom) {
     if (!atom->write && !atom->subatoms) {
