@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2016 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2016 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -29,8 +29,8 @@
 #endif
 #include <stdlib.h>
 #include <math.h>
-#include "../../deadbeef.h"
-#include "../../strdupa.h"
+#include <deadbeef/deadbeef.h>
+#include <deadbeef/strdupa.h>
 #include "aac_parser.h"
 #include "aac_decoder_faad2.h"
 
@@ -39,12 +39,11 @@
 #define min(x,y) ((x)<(y)?(x):(y))
 #define max(x,y) ((x)>(y)?(x):(y))
 
-#define trace(...) { deadbeef->log_detailed (&plugin.plugin, 0, __VA_ARGS__); }
+#define trace(...) { deadbeef->log_detailed (&plugin.decoder.plugin, 0, __VA_ARGS__); }
 
-static DB_decoder_t plugin;
+static ddb_decoder2_t plugin;
 DB_functions_t *deadbeef;
 
-#define OUT_BUFFER_SIZE 1024*8*2 // AAC frame can be 1024 or 960 samples, up to 8 channels, 2 bytes each
 #define AAC_MAX_PACKET_SIZE 768*8 // setting max input packet size, to have some headroom
 
 #define MP4FILE mp4ff_t *
@@ -100,7 +99,8 @@ typedef struct {
     int remaining;
 
     // buffer with decoded samples
-    uint8_t out_buffer[OUT_BUFFER_SIZE];
+    uint8_t *out_buffer;
+    size_t out_buffer_size;
     int out_remaining;
     int num_errors;
     char *samplebuffer;
@@ -110,10 +110,13 @@ typedef struct {
     int junk;
 } aac_info_t;
 
+static int
+aac_seek_sample (DB_fileinfo_t *_info, int sample);
+
 // allocate codec control structure
 static DB_fileinfo_t *
 aac_open (uint32_t hints) {
-    aac_info_t *info = calloc (sizeof (aac_info_t), 1);
+    aac_info_t *info = calloc (1, sizeof (aac_info_t));
     return (DB_fileinfo_t *)info;
 }
 
@@ -124,15 +127,9 @@ parse_aac_stream(DB_FILE *fp, int *psamplerate, int *pchannels, float *pduration
     int64_t firstframepos = -1;
     int64_t fsize = -1;
     int offs = 0;
+
     if (!fp->vfs->is_streaming ()) {
-        int skip = deadbeef->junk_get_leading_size (fp);
-        if (skip >= 0) {
-            deadbeef->fseek (fp, skip, SEEK_SET);
-        }
-        fsize = deadbeef->fgetlength (fp);
-        if (skip > 0) {
-            fsize -= skip;
-        }
+        fsize = deadbeef->fgetlength (fp) - deadbeef->ftell(fp);
     }
 
     uint8_t buf[ADTS_HEADER_SIZE*8];
@@ -218,8 +215,14 @@ parse_aac_stream(DB_FILE *fp, int *psamplerate, int *pchannels, float *pduration
 // returns -1 for error, 0 for aac
 static int
 aac_probe (DB_FILE *fp, float *duration, int *samplerate, int *channels, int64_t *totalsamples) {
-
     deadbeef->rewind (fp);
+    if (!fp->vfs->is_streaming ()) {
+        int skip = deadbeef->junk_get_leading_size (fp);
+        if (skip >= 0) {
+            deadbeef->fseek (fp, skip, SEEK_CUR);
+        }
+    }
+
     if (parse_aac_stream (fp, samplerate, channels, duration, totalsamples) == -1) {
         return -1;
     }
@@ -365,14 +368,14 @@ aac_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     }
 
     _info->fmt.bps = 16;
-    _info->plugin = &plugin;
+    _info->plugin = &plugin.decoder;
 
     if (!info->file->vfs->is_streaming ()) {
         int64_t endsample = deadbeef->pl_item_get_endsample(it);
         if (endsample > 0) {
             info->startsample = deadbeef->pl_item_get_startsample(it);
             info->endsample = endsample;
-            plugin.seek_sample (_info, 0);
+            aac_seek_sample (_info, 0);
         }
         else {
             info->startsample = 0;
@@ -416,6 +419,7 @@ aac_free (DB_fileinfo_t *_info) {
         if (info->dec) {
             aacDecoderClose (info->dec);
         }
+        free (info->out_buffer);
         free (info);
     }
 }
@@ -597,7 +601,7 @@ aac_read (DB_fileinfo_t *_info, char *bytes, int size) {
             trace ("NeAACDecDecode %d bytes\n", info->remaining)
             samples = aacDecoderDecodeFrame (info->dec, &info->frame_info, info->buffer, info->remaining);
             bitrate_bytes += info->frame_info.bytesconsumed;
-            bitrate_samples += info->frame_info.samples / info->frame_info.channels;
+            bitrate_samples += info->frame_info.channels > 0 ? info->frame_info.samples / info->frame_info.channels : 0;
             trace ("samples =%p\n", samples);
             if (!samples) {
 //                trace ("NeAACDecDecode failed with error %s (%d), consumed=%d\n", NeAACDecGetErrorMessage(info->frame_info.error), (int)info->frame_info.error, (int)info->frame_info.bytesconsumed);
@@ -625,6 +629,10 @@ aac_read (DB_fileinfo_t *_info, char *bytes, int size) {
         }
 
         if (info->frame_info.samples > 0) {
+            if (info->out_buffer_size < info->frame_info.samples * 2) {
+                info->out_buffer_size = info->frame_info.samples * 2;
+                info->out_buffer = realloc (info->out_buffer, info->out_buffer_size);
+            }
             memcpy (info->out_buffer, samples, info->frame_info.samples * 2);
             info->out_remaining = (int)(info->frame_info.samples / info->frame_info.channels);
         }
@@ -641,8 +649,8 @@ aac_read (DB_fileinfo_t *_info, char *bytes, int size) {
 }
 
 // returns -1 on error, skipsamples on success
-int
-seek_raw_aac (aac_info_t *info, int sample) {
+int64_t
+seek_raw_aac (aac_info_t *info, int64_t sample) {
     uint8_t buf[ADTS_HEADER_SIZE*8];
 
     int bufsize = 0;
@@ -687,7 +695,7 @@ seek_raw_aac (aac_info_t *info, int sample) {
 }
 
 static int
-aac_seek_sample (DB_fileinfo_t *_info, int sample) {
+aac_seek_sample64 (DB_fileinfo_t *_info, int64_t sample) {
     aac_info_t *info = (aac_info_t *)_info;
 
     sample += info->startsample;
@@ -695,7 +703,7 @@ aac_seek_sample (DB_fileinfo_t *_info, int sample) {
     if (info->mp4file) {
         mp4p_atom_t *stts_atom = mp4p_atom_find(info->trak, "trak/mdia/minf/stbl/stts");
 
-        uint64_t seeksample = (int)((int64_t)sample * info->aac_samplerate / _info->fmt.samplerate);
+        uint64_t seeksample = (int)(sample * info->aac_samplerate / _info->fmt.samplerate);
 
         uint64_t startsample = 0;
         info->mp4sample = mp4p_stts_mp4sample_containing_sample(stts_atom, seeksample, &startsample);
@@ -726,9 +734,14 @@ aac_seek_sample (DB_fileinfo_t *_info, int sample) {
     return 0;
 }
 
+static int
+aac_seek_sample (DB_fileinfo_t *_info, int sample) {
+    return aac_seek_sample64(_info, sample);
+}
+
 static int
 aac_seek (DB_fileinfo_t *_info, float t) {
-    return aac_seek_sample (_info, t * _info->fmt.samplerate);
+    return aac_seek_sample64 (_info, (float)((double)t * (int64_t)_info->fmt.samplerate));
 }
 
 typedef struct {
@@ -885,8 +898,6 @@ _mp4_insert(DB_playItem_t **after, const char *fname, DB_FILE *fp, ddb_playlist_
     mp4_init_ddb_file_callbacks (&info.mp4reader);
     mp4 = info.mp4file = mp4p_open(&info.mp4reader);
 
-    // FIXME: error handling doesn't close the file
-
     if (!mp4) {
         return -1; // not mp4
     }
@@ -900,8 +911,13 @@ _mp4_insert(DB_playItem_t **after, const char *fname, DB_FILE *fp, ddb_playlist_
             mp4p_atom_t *aac_atom = mp4p_atom_find (info.trak, "trak/mdia/minf/stbl/stsd/mp4a");
             if (aac_atom) {
                 aac = aac_atom->data;
-                info.aac_samplerate = aac->sample_rate;
-                break;
+                if (aac->sample_rate != 0) {
+                    info.aac_samplerate = aac->sample_rate;
+                    break;
+                }
+                else {
+                    aac = NULL;
+                }
             }
         }
         info.trak = info.trak->next;
@@ -939,7 +955,7 @@ _mp4_insert(DB_playItem_t **after, const char *fname, DB_FILE *fp, ddb_playlist_
     totalsamples = total_sample_duration * samplerate / info.aac_samplerate;
     duration = total_sample_duration / (float)info.aac_samplerate;
 
-    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.plugin.id);
+    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.decoder.plugin.id);
     ftype = "MP4 AAC";
     deadbeef->pl_add_meta (it, ":FILETYPE", ftype);
     deadbeef->plt_set_item_duration (plt, it, duration);
@@ -952,7 +968,7 @@ _mp4_insert(DB_playItem_t **after, const char *fname, DB_FILE *fp, ddb_playlist_
     int64_t fsize = deadbeef->fgetlength (fp);
 
     char s[100];
-    snprintf (s, sizeof (s), "%lld", fsize);
+    snprintf (s, sizeof (s), "%lld", (long long)fsize);
     deadbeef->pl_add_meta (it, ":FILE_SIZE", s);
     deadbeef->pl_add_meta (it, ":BPS", "16");
     snprintf (s, sizeof (s), "%d", channels);
@@ -1059,7 +1075,7 @@ aac_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
         deadbeef->fclose (fp);
         return NULL;
     }
-    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.plugin.id);
+    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.decoder.plugin.id);
     deadbeef->pl_add_meta (it, ":FILETYPE", ftype);
     deadbeef->plt_set_item_duration (plt, it, duration);
 
@@ -1074,7 +1090,7 @@ aac_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 
     if (duration > 0) {
         char s[100];
-        snprintf (s, sizeof (s), "%lld", fsize);
+        snprintf (s, sizeof (s), "%lld", (long long)fsize);
         deadbeef->pl_add_meta (it, ":FILE_SIZE", s);
         deadbeef->pl_add_meta (it, ":BPS", "16");
         snprintf (s, sizeof (s), "%d", channels);
@@ -1117,19 +1133,20 @@ aac_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 static const char * exts[] = { "aac", "mp4", "m4a", "m4b", NULL };
 
 // define plugin interface
-static DB_decoder_t plugin = {
-    .plugin.api_vmajor = 1,
-    .plugin.api_vminor = 0,
-    .plugin.version_major = 2,
-    .plugin.version_minor = 0,
-//    .plugin.flags = DDB_PLUGIN_FLAG_LOGGING,
-    .plugin.type = DB_PLUGIN_DECODER,
-    .plugin.id = "aac",
-    .plugin.name = "AAC player",
-    .plugin.descr = "plays aac files, supports raw aac files, as well as mp4 container",
-    .plugin.copyright = 
+static ddb_decoder2_t plugin = {
+    .decoder.plugin.api_vmajor = 1,
+    .decoder.plugin.api_vminor = 0,
+    .decoder.plugin.version_major = 2,
+    .decoder.plugin.version_minor = 0,
+//    .decoder.plugin.flags = DDB_PLUGIN_FLAG_LOGGING|DDB_PLUGIN_FLAG_IMPLEMENTS_DECODER2,
+    .decoder.plugin.flags = DDB_PLUGIN_FLAG_IMPLEMENTS_DECODER2,
+    .decoder.plugin.type = DB_PLUGIN_DECODER,
+    .decoder.plugin.id = "aac",
+    .decoder.plugin.name = "AAC player",
+    .decoder.plugin.descr = "plays aac files, supports raw aac files, as well as mp4 container",
+    .decoder.plugin.copyright =
         "AAC DeaDBeeF Player Plugin\n"
-        "Copyright (c) 2009-2020 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (c) 2009-2020 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "This software is provided 'as-is', without any express or implied\n"
         "warranty.  In no event will the authors be held liable for any damages\n"
@@ -1166,17 +1183,18 @@ static DB_decoder_t plugin = {
         "along with this program; if not, write to the Free Software\n"
         "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n"
     ,
-    .plugin.website = "http://deadbeef.sf.net",
-    .open = aac_open,
-    .init = aac_init,
-    .free = aac_free,
-    .read = aac_read,
-    .seek = aac_seek,
-    .seek_sample = aac_seek_sample,
-    .insert = aac_insert,
-    .read_metadata = mp4_read_metadata,
-    .write_metadata = mp4_write_metadata,
-    .exts = exts,
+    .decoder.plugin.website = "http://deadbeef.sf.net",
+    .decoder.open = aac_open,
+    .decoder.init = aac_init,
+    .decoder.free = aac_free,
+    .decoder.read = aac_read,
+    .decoder.seek = aac_seek,
+    .decoder.seek_sample = aac_seek_sample,
+    .decoder.insert = aac_insert,
+    .decoder.read_metadata = mp4_read_metadata,
+    .decoder.write_metadata = mp4_write_metadata,
+    .decoder.exts = exts,
+    .seek_sample64 = aac_seek_sample64,
 };
 
 DB_plugin_t *
