@@ -16,7 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* screwed/maintained by Alexey Yakovenko <waker@users.sourceforge.net> */
+/* screwed/maintained by Oleksiy Yakovenko <waker@users.sourceforge.net> */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,15 +40,15 @@
 #include <cdio/cdtext.h>
 #include <cddb/cddb.h>
 
-#include "../../deadbeef.h"
+#include <deadbeef/deadbeef.h>
 
 //#define trace(...) { fprintf (stderr, __VA_ARGS__); }
 #define trace(fmt,...)
 
 #define CDDA_ALL_TRACKS "all.cda"
 
-#define DEFAULT_SERVER "freedb.org"
-#define DEFAULT_PORT 888
+#define DEFAULT_SERVER "gnudb.gnudb.org"
+#define DEFAULT_PORT 8880
 #define DEFAULT_USE_CDDB 1
 #define DEFAULT_PROTOCOL 1
 #define DEFAULT_PREFER_CDTEXT 1
@@ -88,6 +88,8 @@ struct cddb_thread_params
 {
     DB_playItem_t **items;
     cddb_disc_t *disc;
+    int got_cdtext;
+    int prefer_cdtext;
 };
 
 static cddb_disc_t *
@@ -421,8 +423,13 @@ cleanup_thread_params (struct cddb_thread_params *params)
 }
 
 static void
-replace_meta(DB_playItem_t *it, const char *key, const char *value)
+replace_meta(struct cddb_thread_params *params, DB_playItem_t *it, const char *key, const char *value)
 {
+    if (params && params->got_cdtext && deadbeef->pl_find_meta (it, key) && params->prefer_cdtext) {
+        // already got the preferred value from cdtext
+        return;
+    }
+
     if (value) {
         deadbeef->pl_replace_meta(it, key, value);
     }
@@ -432,18 +439,25 @@ replace_meta(DB_playItem_t *it, const char *key, const char *value)
 }
 
 static void
-write_metadata(DB_playItem_t *it, const cddb_disc_t *disc, const char *num_tracks)
+write_metadata(struct cddb_thread_params *params, DB_playItem_t *it, const cddb_disc_t *disc, const char *num_tracks)
 {
     const int track_nr = deadbeef->pl_find_meta_int(it, "track", 0);
     cddb_track_t *track = cddb_disc_get_track(disc, track_nr-1);
 
-    replace_meta(it, "artist", cddb_disc_get_artist(disc));
-    replace_meta(it, "title", cddb_track_get_title(track));
-    replace_meta(it, "album", cddb_disc_get_title(disc));
-    replace_meta(it, "genre", cddb_disc_get_genre(disc));
+    replace_meta(params, it, "artist", cddb_disc_get_artist(disc));
+    replace_meta(params, it, "title", cddb_track_get_title(track));
+    replace_meta(params, it, "album", cddb_disc_get_title(disc));
+    replace_meta(params, it, "genre", cddb_disc_get_genre(disc));
     const unsigned int year = cddb_disc_get_year(disc);
     year ? deadbeef->pl_set_meta_int(it, "year", year) : deadbeef->pl_delete_meta(it, "year");
-    replace_meta(it, "numtracks", num_tracks);
+    replace_meta(params, it, "numtracks", num_tracks);
+
+    // if the title is still empty, fill with autogenerated title
+    if (!deadbeef->pl_find_meta (it, "title")) {
+        char title[50];
+        snprintf(title, sizeof (title), "CD Track %02d", track_nr);
+        deadbeef->pl_add_meta(it, "title", title);
+    }
 
     ddb_event_track_t *ev = (ddb_event_track_t *)deadbeef->event_alloc(DB_EV_TRACKINFOCHANGED);
     ev->track = it;
@@ -472,7 +486,7 @@ cddb_thread (void *params_void)
     snprintf(num_tracks, sizeof(num_tracks), "%02d", track_count);
     for (size_t i = 0; params->items[i]; i++) {
         deadbeef->pl_add_meta(params->items[i], CDDB_IDS_TAG, disc_list);
-        write_metadata(params->items[i], params->disc, num_tracks);
+        write_metadata(params, params->items[i], params->disc, num_tracks);
     }
 
     cleanup_thread_params(params);
@@ -502,7 +516,10 @@ read_track_cdtext (CdIo_t *cdio, int track_nr, DB_playItem_t *item)
     int field_type;
     for (field_type = 0; field_type < MAX_CDTEXT_FIELDS; field_type++) {
 #if CDIO_API_VERSION >= 6
-        const char *text = cdtext_get_const (cdtext, field_type, track_nr);
+        const char *text = cdtext_get_const (cdtext, field_type, 0);
+        if (!text) {
+            text = cdtext_get_const (cdtext, field_type, track_nr);
+        }
 #else
         const char *text = cdtext_get_const (field_type, cdtext);
 #endif
@@ -521,8 +538,9 @@ read_track_cdtext (CdIo_t *cdio, int track_nr, DB_playItem_t *item)
 
     // some versions of cdio does not convert strings to UTF-8 as documented
     if (album) {
-        const char * album_charset = deadbeef->junk_detect_charset (album);
-        if (album_charset != NULL) {
+        const char *album_charset = deadbeef->junk_detect_charset (album);
+        char *album_converted = NULL;
+        if (album_charset) {
             trace ( "cdda: album field using %s charset, converting\n",album_charset);
             char *album_converted = malloc (strlen(album) * 4);
             if (album_converted) {
@@ -530,15 +548,16 @@ read_track_cdtext (CdIo_t *cdio, int track_nr, DB_playItem_t *item)
                 album = album_converted;
             }
         }
-        replace_meta(item, "album", album);
-        if (album_charset) {
-            free (album);
-            album = NULL;
+        replace_meta(NULL, item, "album", album);
+        if (album_converted) {
+            free (album_converted);
+            album = album_converted = NULL;
         }
     }
 
     if (artist) {
-        const char * artist_charset = deadbeef->junk_detect_charset (artist);
+        const char *artist_charset = deadbeef->junk_detect_charset (artist);
+        char *artist_converted = NULL;
         if (artist_charset != NULL) {
             trace ( "cdda: artist field using %s charset, converting\n",artist_charset);
             char *artist_converted = malloc (strlen(artist) * 4);
@@ -547,11 +566,11 @@ read_track_cdtext (CdIo_t *cdio, int track_nr, DB_playItem_t *item)
                 artist = artist_converted;
             }
         }
-        replace_meta(item, "artist", artist);
+        replace_meta(NULL, item, "artist", artist);
 
         if (artist_charset) {
-            free (artist);
-            artist = NULL;
+            free (artist_converted);
+            artist = artist_converted = NULL;
         }
     }
 
@@ -596,18 +615,21 @@ read_track_cdtext (CdIo_t *cdio, int track_nr, DB_playItem_t *item)
         if (field) {
             // convert to UTF-8 if not UTF-8 already
             const char *text_charset = deadbeef->junk_detect_charset (text);
+            char *text_converted = NULL;
             if (text_charset != NULL) {
                 trace ("cdda: text field using %s charset, converting\n",text_charset);
-                char *text_converted = malloc (strlen(text) * 4);
+                text_converted = malloc (strlen(text) * 4);
                 if (text_converted) {
                     deadbeef->junk_iconv (field, strlen(text), text_converted, strlen(text)*4, text_charset, "UTF-8");
                     text = text_converted;
                 }
             }
             trace("%s: %s\n", field, text);
-            replace_meta(item, field, text);
-            if (text && text_charset != NULL)
-                free (text);
+            replace_meta(NULL, item, field, text);
+            if (text_converted && text_charset != NULL) {
+                free (text_converted);
+                text = text_converted = NULL;
+            }
         }
     }
 }
@@ -643,16 +665,12 @@ insert_track (ddb_playlist_t *plt, DB_playItem_t *after, const char* path, const
         const float sector_count = cdio_get_track_sec_count(cdio, track_nr);
         deadbeef->plt_set_item_duration(plt, it, sector_count / 75);
 
-        char title[12];
-        sprintf(title, "CD Track %02d", track_nr);
-        deadbeef->pl_add_meta(it, "title", title);
-
         char track[4];
-        sprintf(track, "%02d", track_nr);
+        snprintf(track, sizeof (track), "%02d", track_nr);
         deadbeef->pl_add_meta(it, "track", track);
 
         char discid_string[10];
-        sprintf(discid_string, "%08x", discid);
+        snprintf(discid_string, sizeof (discid_string), "%08x", discid);
         deadbeef->pl_add_meta(it, CDDB_DISCID_TAG, discid_string);
 
         it = deadbeef->plt_insert_item(plt, after, it);
@@ -700,13 +718,26 @@ insert_disc (ddb_playlist_t *plt, DB_playItem_t *after, const char *path, const
         const int got_cdtext = read_disc_cdtext(cdio, p->items, tracks);
         const int prefer_cdtext = deadbeef->conf_get_int("cdda.prefer_cdtext", DEFAULT_PREFER_CDTEXT);
         const int enable_cddb = deadbeef->conf_get_int("cdda.freedb.enable", DEFAULT_USE_CDDB);
-        if (!(got_cdtext && prefer_cdtext) && enable_cddb) {
-            trace("cdda: querying freedb...\n");
+        p->got_cdtext = got_cdtext;
+        p->prefer_cdtext = prefer_cdtext;
+        if (enable_cddb) {
             tid = deadbeef->thread_start(cddb_thread, p);
             if (tid) {
                 deadbeef->thread_detach(tid);
             }
         }
+        else {
+            // if cddb is off, fill empty titles with autogenerated ones
+            for (int i = 0; i < item_count; i++) {
+                DB_playItem_t *it = p->items[i];
+                if (!deadbeef->pl_find_meta (it, "title")) {
+                    const int track_nr = deadbeef->pl_find_meta_int(it, "track", 0);
+                    char title[50];
+                    snprintf(title, sizeof (title), "CD Track %02d", track_nr);
+                    deadbeef->pl_add_meta(it, "title", title);
+                }
+            }
+        }
     }
 
     if (!tid) {
@@ -812,13 +843,13 @@ get_param (const char *key, char *value, int len, const char *def)
     return;
 }
 
-#define DRIVE_COMBO_SCRIPT\
+#define DRIVE_COMBO_SCRIPT \
 "property box vbox[1] hmg expand fill border=10 height=250;"\
 "property box hbox[1] hmg height=-1;"\
 "property \"CD drive to load\" select[%u] cdda.drive_device 0"
 
 static int
-cda_action_add_cd (DB_plugin_action_t *act, int ctx)
+cda_action_add_cd (DB_plugin_action_t *act, ddb_action_context_t ctx)
 {
     /* Get all devices containg CD audio media */
     cdio_close_tray(NULL, NULL);
@@ -838,7 +869,7 @@ cda_action_add_cd (DB_plugin_action_t *act, int ctx)
 
         char *layout = malloc(device_combo_length);
         if (layout) {
-            sprintf(layout, DRIVE_COMBO_SCRIPT, device_count);
+            snprintf(layout, device_combo_length, DRIVE_COMBO_SCRIPT, (unsigned)device_count);
             for (char **device = device_list; *device; device++) {
                 strcat(layout, " ");
                 strcat(layout, *device);
@@ -940,7 +971,8 @@ load_cddb_data (ddb_playlist_t *plt, cddb_disc_t *disc, const size_t disc_num)
     snprintf(num_tracks, sizeof(num_tracks), "%02d", track_count);
     do {
         if (deadbeef->pl_is_selected(it)) {
-            write_metadata(it, disc, num_tracks);
+            // FIXME: Reloading from cddb can't combine cdtext with cddb data
+            write_metadata(NULL, it, disc, num_tracks);
         }
         deadbeef->pl_item_unref(it);
         it = deadbeef->pl_get_next(it, PL_MAIN);
@@ -953,7 +985,7 @@ load_cddb_data (ddb_playlist_t *plt, cddb_disc_t *disc, const size_t disc_num)
 }
 
 static int
-action_disc_n (DB_plugin_action_t *act, int ctx)
+action_disc_n (DB_plugin_action_t *act, ddb_action_context_t ctx)
 {
     const int disc_num = atoi(act->name+11);
     int res = -1;
@@ -1079,14 +1111,14 @@ cda_get_actions (DB_playItem_t *it)
 }
 
 static const char settings_dlg[] =
-    "property \"Use CDDB/FreeDB\" checkbox cdda.freedb.enable 1;\n"
+    "property \"Use CDDB/GnuDb\" checkbox cdda.freedb.enable 1;\n"
     "property box hbox[2] height=-1;"
     "property box hbox[0] border=5 height=-1;"
     "property box vbox[4] expand fill height=-1;"
     "property \"Prefer CD-Text over CDDB\" checkbox cdda.prefer_cdtext 1;\n"
-    "property \"CDDB url (e.g. 'freedb.org')\" entry cdda.freedb.host freedb.org;\n"
+    "property \"CDDB url (e.g. 'gnudb.gnudb.org')\" entry cdda.freedb.host gnudb.gnudb.org;\n"
     "property box hbox[1] height=-1;"
-    "property \"CDDB port number (e.g. '888')\" entry cdda.freedb.port 888;\n"
+    "property \"CDDB port number (e.g. '8880')\" entry cdda.freedb.port 8880;\n"
     "property \"Use CDDB protocol\" checkbox cdda.protocol 1;\n"
     "property \"Enable NRG image support\" checkbox cdda.enable_nrg 0;"
     "property box hbox[1] height=-1;"
@@ -1106,7 +1138,7 @@ static DB_decoder_t plugin = {
     .plugin.name = "Audio CD player",
     .plugin.descr = "Audio CD plugin using libcdio and libcddb",
     .plugin.copyright =
-        "Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2009-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "Copyright (C) 2009-2011 Viktor Semykin <thesame.ml@gmail.com>\n"
         "\n"
         "This program is free software; you can redistribute it and/or\n"
