@@ -1,6 +1,6 @@
 /*
     DeaDBeeF - The Ultimate Music Player
-    Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    Copyright (C) 2009-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,8 +24,8 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-#include "../../deadbeef.h"
-#include "../../strdupa.h"
+#include <deadbeef/deadbeef.h>
+#include <deadbeef/strdupa.h>
 #include "dca.h"
 #include "gettimeofday.h"
 
@@ -72,7 +72,7 @@ typedef struct {
 static DB_decoder_t plugin;
 DB_functions_t *deadbeef;
 
-#define BUFFER_SIZE 24576
+#define BUFFER_SIZE 65536
 #define OUT_BUFFER_SIZE 25000 // one block may be up to 22K samples, which is 88Kb for stereo
 #define HEADER_SIZE 14
 typedef struct {
@@ -224,8 +224,6 @@ convert_samples (ddb_dca_state_t *state, int flags)
 {
     sample_t *_samples = dca_samples (state->state);
 
-    int samplesize = state->info.fmt.channels * state->info.fmt.bps / 8;
-
     int n, i, c;
     n = 256;
     int16_t *dst = state->output_buffer + state->remaining * state->info.fmt.channels;
@@ -243,11 +241,11 @@ convert_samples (ddb_dca_state_t *state, int flags)
 
 // returns number of frames decoded
 static int
-dca_decode_data (ddb_dca_state_t *ddb_state, uint8_t * start, int size, int probe)
+dca_decode_data (ddb_dca_state_t *ddb_state, uint8_t * start, size_t size, int probe)
 {
     int n_decoded = 0;
     uint8_t * end = start + size;
-    int len;
+    size_t len;
 
     while (1) {
         len = end - start;
@@ -313,10 +311,9 @@ error:
 }
 
 // returns offset to DTS data in the file, or -1
-static int
+static int64_t
 dts_open_wav (DB_FILE *fp, wavfmt_t *fmt, int64_t *totalsamples) {
     char riff[4];
-    int offset = -1;
 
     if (deadbeef->fread (&riff, 1, sizeof (riff), fp) != sizeof (riff)) {
         return -1;
@@ -398,7 +395,8 @@ dts_open_wav (DB_FILE *fp, wavfmt_t *fmt, int64_t *totalsamples) {
 
 static DB_fileinfo_t *
 dts_open (uint32_t hints) {
-    return calloc (1, sizeof (ddb_dca_state_t));
+    ddb_dca_state_t *info = calloc (1, sizeof (ddb_dca_state_t));
+    return &info->info;
 }
 
 static int
@@ -446,6 +444,10 @@ dts_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         trace ("dca: probe failed\n");
         return -1;
     }
+
+    info->bufptr = info->buf;
+    info->bufpos = info->buf + HEADER_SIZE;
+
     info->frame_byte_size = len;
 
     int flags = info->flags &~ (DCA_LFE | DCA_ADJUST_LEVEL);
@@ -512,6 +514,8 @@ dts_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         info->endsample = totalsamples-1;
     }
 
+    deadbeef->pl_set_meta_int (it, ":CHANNELS", _info->fmt.channels);
+
     trace ("dca_init: nchannels: %d, samplerate: %d\n", _info->fmt.channels, _info->fmt.samplerate);
     return 0;
 }
@@ -536,7 +540,7 @@ dts_read (DB_fileinfo_t *_info, char *bytes, int size) {
     int samplesize = _info->fmt.channels * _info->fmt.bps / 8;
     if (info->endsample >= 0) {
         if (info->currentsample + size / samplesize > info->endsample) {
-            size = (info->endsample - info->currentsample + 1) * samplesize;
+            size = (int)((info->endsample - info->currentsample + 1) * samplesize);
             if (size <= 0) {
                 return 0;
             }
@@ -610,7 +614,7 @@ dts_seek_sample (DB_fileinfo_t *_info, int sample) {
     int64_t offs = info->frame_byte_size * nframe + info->offset;
     deadbeef->fseek (info->file, offs, SEEK_SET);
     info->remaining = 0;
-    info->skipsamples = sample - nframe * info->frame_length;
+    info->skipsamples = (int)(sample - nframe * info->frame_length);
 
     info->currentsample = sample;
     _info->readpos = (float)(sample - info->startsample) / _info->fmt.samplerate;
@@ -619,7 +623,6 @@ dts_seek_sample (DB_fileinfo_t *_info, int sample) {
 
 static int
 dts_seek (DB_fileinfo_t *_info, float time) {
-    ddb_dca_state_t *info = (ddb_dca_state_t *)_info;
     return dts_seek_sample (_info, time * _info->fmt.samplerate);
 }
 
@@ -638,7 +641,7 @@ dts_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     int64_t totalsamples = -1;
     const char *filetype = NULL;
 
-    int offset = 0;
+    int64_t offset = 0;
     double dur = -1;
     // WAV format
     if ((offset = dts_open_wav (fp, &fmt, &totalsamples)) != -1) {
@@ -711,6 +714,9 @@ dts_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     return after;
 error:
     if (state) {
+        if (state->state) {
+            dca_free(state->state);
+        }
         free (state);
     }
     if (fp) {
@@ -739,7 +745,7 @@ static DB_decoder_t plugin = {
     .plugin.name = "dts decoder",
     .plugin.descr = "plays dts-encoded files using libdca from VLC project",
     .plugin.copyright = 
-        "Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2009-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "Uses modified libdca from VLC Player project,\n"
         "developed by Gildas Bazin <gbazin@videolan.org>"
