@@ -1,6 +1,6 @@
 /*
     gtkui hotkey handlers
-    Copyright (C) 2009-2013 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2013 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -24,24 +24,28 @@
 #  include <config.h>
 #endif
 
+#include <gio/gio.h>
+#include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtk.h>
+#include <sys/stat.h>
 #include <unistd.h>
+#include <deadbeef/deadbeef.h>
 #include "../../gettext.h"
-#include "../../deadbeef.h"
+#include "../../shared/deletefromdisk.h"
+#include "callbacks.h"
+#include "playlist/ddblistview.h"
 #include "gtkui.h"
+#include "gtkui_api.h"
+#include "gtkui_deletefromdisk.h"
+#include "interface.h"
+#include "prefwin/prefwin.h"
 #include "progress.h"
-#include "ddblistview.h"
 #include "search.h"
 #include "support.h"
-#include "wingeom.h"
-#include "interface.h"
 #include "trkproperties.h"
-#include "callbacks.h"
-#include <sys/stat.h>
-#include "gtkui_api.h"
-#include <gio/gio.h>
+#include "wingeom.h"
+#include "undostack.h"
 
 // disable custom title function, until we have new title formatting (0.7)
 #define DISABLE_CUSTOM_TITLE
@@ -149,7 +153,6 @@ action_new_playlist_handler_cb (void *user_data) {
     int pl = gtkui_add_new_playlist ();
     if (pl != -1) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return FALSE;
 }
@@ -164,7 +167,16 @@ int
 action_remove_current_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     int idx = deadbeef->plt_get_curr_idx ();
     if (idx != -1) {
-        deadbeef->plt_remove (idx);
+        gtkui_remove_playlist_at_index (idx);
+    }
+    return 0;
+}
+
+int
+action_rename_current_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
+    int idx = deadbeef->plt_get_curr_idx ();
+    if (idx != -1) {
+        gtkui_rename_playlist_at_index(idx);
     }
     return 0;
 }
@@ -264,26 +276,18 @@ action_add_location_handler_cb (void *user_data) {
         if (entry) {
             const char *text = gtk_entry_get_text (entry);
             if (text) {
-                ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-                if (!deadbeef->plt_add_files_begin (plt, 0)) {
-                    DB_playItem_t *tail = deadbeef->plt_get_last (plt, PL_MAIN);
-                    DB_playItem_t *it = deadbeef->plt_insert_file2 (0, plt, tail, text, NULL, NULL, NULL);
+                char *text_copy = strdup(text);
+                char *trimmed_text = gtkui_trim_whitespace(text_copy, strlen(text_copy));
+
+                const char *custom_title = NULL;
 #ifndef DISABLE_CUSTOM_TITLE
-                    if (it && deadbeef->conf_get_int ("gtkui.location_set_custom_title", 0)) {
-                        deadbeef->pl_replace_meta (it, ":CUSTOM_TITLE", gtk_entry_get_text (GTK_ENTRY (ct)));
-                    }
-#else
-#   pragma unused(it)
+                if (deadbeef->conf_get_int ("gtkui.location_set_custom_title", 0)) {
+                    custom_title = gtk_entry_get_text (GTK_ENTRY (ct));
+                }
 #endif
-                    if (tail) {
-                        deadbeef->pl_item_unref (tail);
-                    }
-                    deadbeef->plt_add_files_end (plt, 0);
-                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-                }
-                if (plt) {
-                    deadbeef->plt_unref (plt);
-                }
+                gtkui_add_location(trimmed_text, custom_title);
+
+                free (text_copy);
             }
         }
     }
@@ -343,9 +347,9 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, ddb_action_context
         deadbeef->plt_unref (plt_curr);
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-        deadbeef->pl_lock ();
-        DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+        DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
         if (it) {
+            deadbeef->pl_lock ();
             ddb_playlist_t *plt = deadbeef->plt_get_curr ();
             if (plt) {
                 int idx = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
@@ -356,194 +360,51 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, ddb_action_context
                 }
                 deadbeef->plt_unref (plt);
             }
+            deadbeef->pl_unlock ();
             deadbeef->pl_item_unref (it);
         }
-        deadbeef->pl_unlock ();
     }
     return 0;
 }
 
-static void
-_remove_file_from_all_playlists (const char *search_uri) {
-    // The caller is responsible for pl_lock
-    int n = deadbeef->plt_get_count ();
-    for (int i = 0; i < n; ++i) {
-        ddb_playlist_t *plt = deadbeef->plt_get_for_idx (i);
-        DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
-        while (it) {
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            const char *uri = deadbeef->pl_find_meta (it, ":URI");
-            if (strcmp (uri, search_uri) == 0) {
-                deadbeef->plt_remove_item (plt, it);
-            }
-            deadbeef->pl_item_unref (it);
-            it = next;
-        }
 
-        deadbeef->plt_unref (plt);
-    }
-}
+static ddbDeleteFromDiskController_t _deleteCtl;
 
 static void
-_delete_and_remove_track_from_all_playlists (const char *uri, ddb_playlist_t *plt, ddb_playItem_t *it) {
-
-    if (deadbeef->conf_get_int ("gtkui.move_to_trash", 1)) {
-        GFile *file = g_file_new_for_path (uri);
-        g_file_trash (file, NULL, NULL);
-        g_object_unref (file);
-    } else {
-        (void)unlink (uri);
-    }
-
-    // check if file exists
-    struct stat buf;
-    memset (&buf, 0, sizeof (buf));
-    int stat_res = stat (uri, &buf);
-    
-    if (stat_res != 0) {
-        _remove_file_from_all_playlists (uri);
-    } else {
-        trace ("Failed to delete file: %s\n", uri);
-    }
+_deleteCompleted (ddbDeleteFromDiskController_t ctl, int cancelled) {
+    ddbDeleteFromDiskControllerFree(ctl);
+    _deleteCtl = NULL;
 }
 
 gboolean
 action_delete_from_disk_handler_cb (void *data) {
+    if (_deleteCtl) {
+        return FALSE;
+    }
+
+    ddbDeleteFromDiskControllerDelegate_t delegate = {
+        .warningMessageForCtx = gtkui_warning_message_for_ctx,
+        .deleteFile = gtkui_delete_file,
+        .completed = _deleteCompleted,
+    };
+
     ddb_playlist_t *plt = deadbeef->plt_get_curr ();
     if (!plt) {
         return FALSE;
     }
-    
+
     ddb_action_context_t ctx = (int)(intptr_t)data;
-    if (deadbeef->conf_get_int ("gtkui.delete_files_ask", 1)) {
-        char buf[1000];
-        const char *buf2 = deadbeef->conf_get_int ("gtkui.move_to_trash", 1) ?
-        _(" The files will be moved to trash.\n\n(This dialog can be turned off in GTKUI plugin settings)") : 
-        _(" The files will be lost.\n\n(This dialog can be turned off in GTKUI plugin settings)");
 
-        if (ctx == DDB_ACTION_CTX_SELECTION) {
-            int selected_files = deadbeef->pl_getselcount ();
-            if (selected_files == 1) {
-                snprintf(buf, sizeof (buf), _("Do you really want to delete the selected file?%s"), buf2);
-            } else {
-                snprintf(buf, sizeof (buf), _("Do you really want to delete all %d selected files?%s"), selected_files, buf2);
-            }
-        }
-        else if (ctx == DDB_ACTION_CTX_PLAYLIST) {
-            int files = deadbeef->plt_get_item_count (plt, PL_MAIN);
-            snprintf(buf, sizeof (buf), _("Do you really want to delete all %d files from the current playlist?%s"), files, buf2);
-        }
-        else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-            snprintf(buf, sizeof (buf), _("Do you really want to delete the currently playing file?%s"), buf2);
-        }
+    _deleteCtl = ddbDeleteFromDiskControllerInitWithPlaylist(ddbDeleteFromDiskControllerAlloc(), plt, ctx);
 
-        GtkWidget *dlg = gtk_message_dialog_new (GTK_WINDOW (mainwin), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, _("Delete files from disk"));
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), "%s", buf);
-        gtk_window_set_title (GTK_WINDOW (dlg), _("Warning"));
+    ddbDeleteFromDiskControllerSetShouldSkipDeletedTracks(_deleteCtl, deadbeef->conf_get_int ("gtkui.skip_deleted_songs", 0));
 
-        int response = gtk_dialog_run (GTK_DIALOG (dlg));
-        gtk_widget_destroy (dlg);
-        if (response != GTK_RESPONSE_YES) {
-            return FALSE;
-        }
-    }
-    deadbeef->pl_lock ();
+    ddbDeleteFromDiskControllerRunWithDelegate(_deleteCtl, delegate);
 
-    DB_playItem_t **tracklist = NULL;
-    unsigned trackcount = 0;
-    
-    DB_playItem_t *it_current_song = deadbeef->streamer_get_playing_track ();
-    int idx_current_song = -1;
-    if (ctx == DDB_ACTION_CTX_SELECTION) {
-        unsigned selcount = deadbeef->plt_getselcount (plt);
-        tracklist = calloc (selcount, sizeof (DB_playItem_t *));
-        DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
-        while (it) {
-            if (trackcount == selcount) {
-                break;
-            }
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            const char *uri = deadbeef->pl_find_meta (it, ":URI");
-            if (deadbeef->pl_is_selected (it) && deadbeef->is_local_file (uri)) {
-                if (it == it_current_song) {
-                    idx_current_song = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
-                }
-                deadbeef->pl_item_ref (it);
-                tracklist[trackcount++] = it;
-            }
-            deadbeef->pl_item_unref (it);
-            it = next;
-        }
-    }
-    else if (ctx == DDB_ACTION_CTX_PLAYLIST) {
-        unsigned count = deadbeef->plt_get_item_count (plt, PL_MAIN);
-        tracklist = calloc (count, sizeof (DB_playItem_t *));
-        DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
-        while (it) {
-            if (trackcount == count) {
-                break;
-            }
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            const char *uri = deadbeef->pl_find_meta (it, ":URI");
-            if (deadbeef->is_local_file (uri)) {
-                deadbeef->pl_item_ref (it);
-                tracklist[trackcount++] = it;
-            }
-            deadbeef->pl_item_unref (it);
-            it = next;
-        }
-    }
-    else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-        DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
-        if (it) {
-            const char *uri = deadbeef->pl_find_meta (it, ":URI");
-            if (deadbeef->is_local_file (uri)) {
-                int idx = idx_current_song = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
-                if (idx != -1) {
-                    tracklist = calloc (1, sizeof (DB_playItem_t *));
-                    deadbeef->pl_item_ref (it);
-                    tracklist[trackcount++] = it;
-                }
-            }
-            deadbeef->pl_item_unref (it);
-        }
-    }
-
-    if (tracklist) {
-        for (unsigned i = 0; i < trackcount; i++) {
-            const char *uri = deadbeef->pl_find_meta (tracklist[i], ":URI");
-            _delete_and_remove_track_from_all_playlists (uri, plt, tracklist[i]);
-            deadbeef->pl_item_unref (tracklist[i]);
-        }
-        free (tracklist);
-        tracklist = NULL;
-    }
-    
-    if (deadbeef->conf_get_int ("gtkui.skip_deleted_songs", 0) 
-        && deadbeef->plt_get_item_idx (plt, it_current_song, PL_MAIN) == -1 
-        && deadbeef->streamer_get_current_playlist () == deadbeef->plt_get_curr_idx () 
-        && deadbeef->get_output ()->state () == OUTPUT_STATE_PLAYING) {
-        
-        if (idx_current_song != -1 
-            && deadbeef->playqueue_get_count () == 0 
-            && deadbeef->streamer_get_shuffle () == DDB_SHUFFLE_OFF) {
-            deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, idx_current_song, 0);
-        }
-        else {
-            deadbeef->sendmessage(DB_EV_NEXT, 0, 0, 0);
-        }
-    }
-
-    deadbeef->pl_save_all ();
-    if (it_current_song) {
-        deadbeef->pl_item_unref (it_current_song);
-    }
-    deadbeef->pl_unlock ();
     deadbeef->plt_unref (plt);
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+
     return FALSE;
 }
-
 int
 action_delete_from_disk_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_delete_from_disk_handler_cb, (void *)(intptr_t)ctx);
@@ -735,7 +596,7 @@ action_toggle_designmode_handler (DB_plugin_action_t *act, ddb_action_context_t
 
 gboolean
 action_preferences_handler_cb (void *data) {
-    gtkui_run_preferences_dlg ();
+    prefwin_run (-1);
     return FALSE;
 }
 
@@ -751,29 +612,38 @@ action_sort_custom_handler_cb (void *data) {
     gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
 
     GtkComboBox *combo = GTK_COMBO_BOX (lookup_widget (dlg, "sortorder"));
-    GtkEntry *entry = GTK_ENTRY (lookup_widget (dlg, "sortfmt"));
+    GtkTextView *sortfmt = GTK_TEXT_VIEW (lookup_widget (dlg, "sortfmt"));
+    GtkTextBuffer *sortbuffer = gtk_text_view_get_buffer(sortfmt);
 
     gtk_combo_box_set_active (combo, deadbeef->conf_get_int ("gtkui.sortby_order", 0));
     deadbeef->conf_lock ();
-    gtk_entry_set_text (entry, deadbeef->conf_get_str_fast ("gtkui.sortby_fmt_v2", ""));
+    const char* fmt = deadbeef->conf_get_str_fast ("gtkui.sortby_fmt_v2", "");
+    gtk_text_buffer_set_text (sortbuffer, fmt, (gint)strlen(fmt));
     deadbeef->conf_unlock ();
 
     int r = gtk_dialog_run (GTK_DIALOG (dlg));
 
     if (r == GTK_RESPONSE_OK) {
         GtkComboBox *combo = GTK_COMBO_BOX (lookup_widget (dlg, "sortorder"));
-        GtkEntry *entry = GTK_ENTRY (lookup_widget (dlg, "sortfmt"));
+        GtkTextView *sortfmt = GTK_TEXT_VIEW (lookup_widget (dlg, "sortfmt"));
+        GtkTextBuffer *sortbuffer = gtk_text_view_get_buffer(sortfmt);
         int order = gtk_combo_box_get_active (combo);
-        const char *fmt = gtk_entry_get_text (entry);
+        GtkTextIter start, end;
+        gtk_text_buffer_get_start_iter(sortbuffer, &start);
+        gtk_text_buffer_get_end_iter(sortbuffer, &end);
+        char *fmt = gtk_text_buffer_get_text (sortbuffer, &start, &end, FALSE);
 
         deadbeef->conf_set_int ("gtkui.sortby_order", order);
         deadbeef->conf_set_str ("gtkui.sortby_fmt_v2", fmt);
 
+
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
         deadbeef->plt_sort_v2 (plt, PL_MAIN, -1, fmt, order == 0 ? DDB_SORT_ASCENDING : DDB_SORT_DESCENDING);
         deadbeef->plt_save_config (plt);
         deadbeef->plt_unref (plt);
 
+        free( fmt);
+
         deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     }
 
@@ -987,3 +857,15 @@ action_toggle_logwindow_handler(DB_plugin_action_t *act, ddb_action_context_t ct
     g_idle_add (action_toggle_logwindow_handler_cb, NULL);
     return 0;
 }
+
+int
+action_undo(DB_plugin_action_t *act, ddb_action_context_t ctx) {
+    gtkui_undostack_perform_undo();
+    return 0;
+}
+
+int
+action_redo(DB_plugin_action_t *act, ddb_action_context_t ctx) {
+    gtkui_undostack_perform_redo();
+    return 0;
+}
