@@ -1,6 +1,6 @@
 /*
     gtkui hotkey handlers
-    Copyright (C) 2009-2013 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2013 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -24,6 +24,8 @@
 #ifndef __ACTIONHANDLERS_H
 #define __ACTIONHANDLERS_H
 
+#include "../../shared/deletefromdisk.h"
+
 gboolean
 action_open_files_handler_cb (void *userdata);
 
@@ -69,6 +71,9 @@ action_new_playlist_handler (struct DB_plugin_action_s *action, ddb_action_conte
 int
 action_remove_current_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
+int
+action_rename_current_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
+
 gboolean
 action_toggle_mainwin_handler_cb (void *user_data);
 
@@ -213,4 +218,10 @@ action_playback_loop_cycle_handler(DB_plugin_action_t *act, ddb_action_context_t
 int
 action_toggle_logwindow_handler(DB_plugin_action_t *act, ddb_action_context_t ctx);
 
+int
+action_undo(DB_plugin_action_t *act, ddb_action_context_t ctx);
+
+int
+action_redo(DB_plugin_action_t *act, ddb_action_context_t ctx);
+
 #endif
