@@ -17,7 +17,10 @@
  * MA 02110-1301 USA
  */
 
+#include <gtk/gtk.h>
 #include <math.h>
+#include <string.h>
+#include "support.h"
 #include "ddb_splitter.h"
 
 /**
@@ -475,7 +478,6 @@ static void
 update_drag (DdbSplitter *splitter)
 {
     gint pos;
-    gint handle_size;
     gint size;
 
     if (splitter->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -492,11 +494,11 @@ update_drag (DdbSplitter *splitter)
     if (size != splitter->priv->child1_size) {
         if (splitter->priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
             float gripsize = (float)splitter->priv->handle_pos.width/a.width;
-            ddb_splitter_set_proportion (splitter, CLAMP ((float)size/a.width, gripsize, 1-gripsize*2));
+            ddb_splitter_set_proportion (splitter, CLAMP ((float)size/a.width, 0, 1-gripsize));
         }
         else {
             float gripsize = (float)splitter->priv->handle_pos.height/a.height;
-            ddb_splitter_set_proportion (splitter, CLAMP ((float)size/a.height, gripsize, 1-gripsize*2));
+            ddb_splitter_set_proportion (splitter, CLAMP ((float)size/a.height, 0, 1-gripsize));
         }
     }
 }
@@ -639,9 +641,10 @@ ddb_splitter_realize (GtkWidget *widget)
             gdk_cursor_unref (attributes.cursor);
         }
 
-        gtk_widget_style_attach (widget);
-        //gtk_style_attach (widget, parent);
-        //widget->style = gtk_style_attach (widget->style, widget->window);
+
+#if !GTK_CHECK_VERSION(3,0,0)
+        widget->style = gtk_style_attach (widget->style, widget->window);
+#endif
 
         if (ddb_splitter_children_visible (splitter)) {
             gdk_window_show (splitter->priv->handle);
@@ -945,7 +948,9 @@ ddb_splitter_size_allocate (GtkWidget *widget, GtkAllocation *a_con)
     GdkRectangle old_handle_pos = sp_priv->handle_pos;
 
     GtkAllocation a_c1;
+    memset (&a_c1, 0, sizeof (a_c1));
     GtkAllocation a_c2;
+    memset (&a_c2, 0, sizeof (a_c2));
     if (sp_priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
         if (c1_visible) {
             // use full height in horitzontal splitter
