@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2015 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -35,15 +35,24 @@ G_BEGIN_DECLS
 #define DDB_IS_VOLUMEBAR_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), DDB_TYPE_VOLUMEBAR))
 #define DDB_VOLUMEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DDB_TYPE_VOLUMEBAR, DdbVolumeBarClass))
 
+typedef struct _DdbVolumeBarPrivate DdbVolumeBarPrivate;
 typedef struct _DdbVolumeBar DdbVolumeBar;
 typedef struct _DdbVolumeBarClass DdbVolumeBarClass;
 
 typedef void * DdbVolumeBarIter;
 typedef void * DdbVolumeBarColIter;
 
+typedef enum {
+    DDB_VOLUMEBAR_SCALE_DB,
+    DDB_VOLUMEBAR_SCALE_LINEAR,
+    DDB_VOLUMEBAR_SCALE_CUBIC,
+    DDB_VOLUMEBAR_SCALE_COUNT
+} DdbVolumeBarScale;
+
 struct _DdbVolumeBar {
     GtkWidget parent;
     int show_dbs;
+    DdbVolumeBarPrivate *priv;
 };
 
 struct _DdbVolumeBarClass {
@@ -56,6 +65,15 @@ GtkWidget * ddb_volumebar_new(void);
 void
 ddb_volumebar_init_signals (DdbVolumeBar *vb, GtkWidget *evbox);
 
+DdbVolumeBarScale
+ddb_volumebar_get_scale (const DdbVolumeBar *volumebar);
+
+void
+ddb_volumebar_set_scale (DdbVolumeBar *volumebar, DdbVolumeBarScale scale);
+
+void
+ddb_volumebar_update(DdbVolumeBar *volumebar);
+
 G_END_DECLS
 
 #endif // __DDBVOLUMEBAR_H
