@@ -1,6 +1,6 @@
 /*
     hotkey action handlers for deadbeef hotkeys plugin
-    Copyright (C) 2009-2013 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2013 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -26,7 +26,7 @@
 
 #include <string.h>
 #include "../../gettext.h"
-#include "../../deadbeef.h"
+#include <deadbeef/deadbeef.h>
 
 extern DB_functions_t *deadbeef;
 
@@ -37,8 +37,8 @@ action_jump_to_current_handler (DB_plugin_action_t *act, ddb_action_context_t ct
 }
 
 static DB_playItem_t*
-skip_to_get_track_helper () {
-    DB_playItem_t *current = deadbeef->streamer_get_playing_track ();
+skip_to_get_track_helper (void) {
+    DB_playItem_t *current = deadbeef->streamer_get_playing_track_safe ();
     if (!current) {
         return NULL;
     }
@@ -369,7 +369,6 @@ action_next_playlist_handler (DB_plugin_action_t *act, ddb_action_context_t ctx)
     }
 
     deadbeef->plt_set_curr_idx (tab);
-    deadbeef->conf_set_int ("playlist.current", tab);
 
     return 0;
 }
@@ -386,7 +385,6 @@ action_prev_playlist_handler (DB_plugin_action_t *act, ddb_action_context_t ctx)
     }
 
     deadbeef->plt_set_curr_idx (tab);
-    deadbeef->conf_set_int ("playlist.current", tab);
 
     return 0;
 }
@@ -396,7 +394,6 @@ action_playlist1_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 0;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -406,7 +403,6 @@ action_playlist2_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 1;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -416,7 +412,6 @@ action_playlist3_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 2;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -426,7 +421,6 @@ action_playlist4_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 3;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -436,7 +430,6 @@ action_playlist5_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 4;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -446,7 +439,6 @@ action_playlist6_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 5;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -456,7 +448,6 @@ action_playlist7_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 6;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -466,7 +457,6 @@ action_playlist8_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 7;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -476,7 +466,6 @@ action_playlist9_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 8;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -486,7 +475,6 @@ action_playlist10_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     int pl = 9;
     if (pl < deadbeef->plt_get_count ()) {
         deadbeef->plt_set_curr_idx (pl);
-        deadbeef->conf_set_int ("playlist.current", pl);
     }
     return 0;
 }
@@ -597,6 +585,11 @@ action_toggle_in_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_
 
 int
 action_move_tracks_up_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
+    ddb_playItem_t *playing_track = NULL;
+    if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
+        playing_track = deadbeef->streamer_get_playing_track_safe ();
+    }
+
     deadbeef->pl_lock ();
 
     ddb_playlist_t *plt = deadbeef->action_get_playlist ();
@@ -628,29 +621,37 @@ action_move_tracks_up_handler (DB_plugin_action_t *act, ddb_action_context_t ctx
         }
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-        it = deadbeef->streamer_get_playing_track ();
-        if (it) {
+        if (playing_track) {
             it_count = 1;
             uint32_t indexes[1];
-            indexes[0] = deadbeef->pl_get_idx_of (it);
-            DB_playItem_t *drop_before = deadbeef->pl_get_prev (it,PL_MAIN);
+            indexes[0] = deadbeef->pl_get_idx_of (playing_track);
+            DB_playItem_t *drop_before = deadbeef->pl_get_prev (playing_track,PL_MAIN);
             if (drop_before) {
                 deadbeef->plt_move_items (plt, PL_MAIN, plt, drop_before, indexes, it_count);
                 deadbeef->pl_item_unref (drop_before);
             }
-            deadbeef->pl_item_unref (it);
         }
     }
 
     deadbeef->plt_save_config (plt);
     deadbeef->plt_unref (plt);
     deadbeef->pl_unlock ();
+
+    if (playing_track != NULL) {
+        deadbeef->pl_item_unref (playing_track);
+    }
+
     deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     return 0;
 }
 
 int
 action_move_tracks_down_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
+    ddb_playItem_t *playing_track = NULL;
+    if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
+        playing_track = deadbeef->streamer_get_playing_track_safe ();
+    }
+
     deadbeef->pl_lock ();
 
     ddb_playlist_t *plt = deadbeef->action_get_playlist ();
@@ -681,22 +682,25 @@ action_move_tracks_down_handler (DB_plugin_action_t *act, ddb_action_context_t c
         }
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-        it = deadbeef->streamer_get_playing_track ();
-        if (it) {
+        if (playing_track) {
             it_count = 1;
             uint32_t indexes[1];
-            indexes[0] = deadbeef->pl_get_idx_of (it);
+            indexes[0] = deadbeef->pl_get_idx_of (playing_track);
             DB_playItem_t *drop_before = deadbeef->pl_get_for_idx (indexes[0]+2);
             deadbeef->plt_move_items (plt, PL_MAIN, plt, drop_before, indexes, it_count);
             if (drop_before)
                 deadbeef->pl_item_unref (drop_before);
-            deadbeef->pl_item_unref (it);
         }
     }
 
     deadbeef->plt_save_config (plt);
     deadbeef->plt_unref (plt);
     deadbeef->pl_unlock ();
+
+    if (playing_track != NULL) {
+        deadbeef->pl_item_unref (playing_track);
+    }
+
     deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     return 0;
 }
@@ -720,6 +724,24 @@ action_add_to_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t c
     return 0;
 }
 
+int
+action_prepend_to_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
+    ddb_playlist_t *plt = deadbeef->action_get_playlist ();
+
+    DB_playItem_t *it = deadbeef->plt_get_last (plt, PL_MAIN);
+    while (it) {
+        if (ctx == DDB_ACTION_CTX_PLAYLIST || (ctx == DDB_ACTION_CTX_SELECTION && deadbeef->pl_is_selected (it))) {
+            deadbeef->playqueue_insert_at (0, it);
+        }
+        DB_playItem_t *prev = deadbeef->pl_get_prev (it, PL_MAIN);
+        deadbeef->pl_item_unref (it);
+        it = prev;
+    }
+
+    deadbeef->plt_unref (plt);
+    return 0;
+}
+
 int
 action_remove_from_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     ddb_playlist_t *plt = deadbeef->action_get_playlist ();
@@ -744,3 +766,106 @@ action_toggle_mute_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     deadbeef->audio_set_mute (mute);
     return 0;
 }
+
+int
+action_prev_or_restart_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
+    if (it) {
+        float dur = deadbeef->pl_get_item_duration (it);
+        deadbeef->pl_item_unref (it);
+        if (dur > 0) {
+            float pos = deadbeef->streamer_get_playpos ();
+            if (pos > 3) {
+                deadbeef->sendmessage (DB_EV_SEEK, 0, 0, 0);
+                return 0;
+            }
+        }
+    }
+    deadbeef->sendmessage (DB_EV_PREV, 0, 0, 0);
+    return 0;
+}
+
+static void
+_copy_playlist_int (ddb_playlist_t *src, ddb_playlist_t *dst) {
+    deadbeef->pl_lock ();
+    DB_playItem_t *it = deadbeef->plt_get_first (src, PL_MAIN);
+    DB_playItem_t *after = NULL;
+    while (it) {
+        DB_playItem_t *it_new = deadbeef->pl_item_alloc ();
+        deadbeef->pl_item_copy (it_new, it);
+        deadbeef->plt_insert_item (dst, after, it_new);
+
+        DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+        if (after) {
+            deadbeef->pl_item_unref (after);
+        }
+        after = it_new;
+        deadbeef->pl_item_unref (it);
+        it = next;
+    }
+    if (after) {
+        deadbeef->pl_item_unref (after);
+    }
+    deadbeef->pl_unlock ();
+    deadbeef->plt_save_config (dst);
+}
+
+static int
+_copy_playlist (ddb_playlist_t *plt) {
+    char orig_title[100];
+    deadbeef->plt_get_title (plt, orig_title, sizeof (orig_title));
+
+    int cnt = deadbeef->plt_get_count ();
+    int i;
+    int idx = 0;
+    for (;;) {
+        char name[100];
+        if (!idx) {
+            snprintf (name, sizeof (name), _("Copy of %s"), orig_title);
+        }
+        else {
+            snprintf (name, sizeof (name), _("Copy of %s (%d)"), orig_title, idx);
+        }
+        deadbeef->pl_lock ();
+        for (i = 0; i < cnt; i++) {
+            char t[100];
+            ddb_playlist_t *plt = deadbeef->plt_get_for_idx (i);
+            deadbeef->plt_get_title (plt, t, sizeof (t));
+            deadbeef->plt_unref (plt);
+            if (!strcasecmp (t, name)) {
+                break;
+            }
+        }
+        deadbeef->pl_unlock ();
+        if (i == cnt) {
+            int new_plt_idx = deadbeef->plt_add (cnt, name);
+            if (new_plt_idx < 0) {
+                return -1;
+            }
+            ddb_playlist_t *new_plt = deadbeef->plt_get_for_idx (new_plt_idx);
+            if (!new_plt) {
+                return -1;
+            }
+            _copy_playlist_int(plt, new_plt);
+            return new_plt_idx;
+        }
+        idx++;
+    }
+    return -1;
+}
+
+
+int
+action_duplicate_playlist_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
+    ddb_playlist_t *plt = deadbeef->action_get_playlist ();
+    if (plt == NULL) {
+        return -1;
+    }
+
+    int playlist = _copy_playlist (plt);
+    if (playlist != -1) {
+        deadbeef->plt_set_curr_idx (playlist);
+    }
+    deadbeef->plt_unref (plt);
+    return 0;
+}
