@@ -1,6 +1,6 @@
 /*
     hotkey action handlers for deadbeef hotkeys plugin
-    Copyright (C) 2009-2013 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2013 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -126,10 +126,19 @@ action_move_tracks_down_handler (DB_plugin_action_t *act, ddb_action_context_t c
 int
 action_add_to_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t ctx);
 
+int
+action_prepend_to_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t ctx);
+
 int
 action_remove_from_playqueue_handler (DB_plugin_action_t *act, ddb_action_context_t ctx);
 
 int
 action_toggle_mute_handler (DB_plugin_action_t *act, ddb_action_context_t ctx);
 
+int
+action_prev_or_restart_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
+
+int
+action_duplicate_playlist_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
+
 #endif
