@@ -1,6 +1,6 @@
 /*
     SID plugin for DeaDBeeF Player
-    Copyright (C) 2009-2014 Alexey Yakovenko <waker@users.sourceforge.net>
+    Copyright (C) 2009-2014 Oleksiy Yakovenko <waker@users.sourceforge.net>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include "../../deadbeef.h"
+#include <deadbeef/deadbeef.h>
 #include "csid.h"
 
 static const char *exts[] = { "sid", NULL };
@@ -24,8 +24,8 @@ static const char settings_dlg[] =
     "property \"Full path to Songlengths.md5/.txt\" file hvsc_path \"\";\n"
     "property \"Samplerate\" entry sid.samplerate 44100;\n"
     "property \"Bits per sample\" select[2] sid.bps 0 16 8;\n"
-    "property \"Mono synth\" checkbox sid.mono 0;\n"
     "property \"Default song length (sec)\" entry sid.defaultlength 180;\n"
+    "property \"Mono synth\" checkbox sid.mono 0;\n"
 ;
 
 // define plugin interface
@@ -39,7 +39,7 @@ DB_decoder_t sid_plugin = {
     .plugin.descr = "SID player based on libsidplay2",
     .plugin.copyright = 
         "SID plugin for DeaDBeeF Player\n"
-        "Copyright (C) 2009-2014 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2009-2014 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "This program is free software: you can redistribute it and/or modify\n"
         "it under the terms of the GNU General Public License as published by\n"
